package com.jiuqi.util.csv;

package .tmp;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.io.Writer;
import java.nio.charset.Charset;

public class CsvWriter {
  private PrintWriter outputStream = null;
  
  private String fileName = null;
  
  private boolean firstColumn = true;
  
  private boolean useCustomRecordDelimiter = false;
  
  private Charset charset = null;
  
  private UserSettings userSettings = new UserSettings(this);
  
  private boolean initialized = false;
  
  private boolean closed = false;
  
  public static final int ESCAPE_MODE_DOUBLED = 1;
  
  public static final int ESCAPE_MODE_BACKSLASH = 2;
  
  public CsvWriter(String fileName, char delimiter, Charset charset) {
    if (fileName == null)
      throw new IllegalArgumentException(
          "Parameter fileName can not be null."); 
    if (charset == null)
      throw new IllegalArgumentException(
          "Parameter charset can not be null."); 
    this.fileName = fileName;
    this.userSettings.Delimiter = delimiter;
    this.charset = charset;
  }
  
  public CsvWriter(String fileName) {
    this(fileName, ',', Charset.forName("ISO-8859-1"));
  }
  
  public CsvWriter(Writer outputStream, char delimiter) {
    if (outputStream == null)
      throw new IllegalArgumentException(
          "Parameter outputStream can not be null."); 
    this.outputStream = new PrintWriter(outputStream);
    this.userSettings.Delimiter = delimiter;
    this.initialized = true;
  }
  
  public CsvWriter(OutputStream outputStream, char delimiter, Charset charset) {
    this(new OutputStreamWriter(outputStream, charset), delimiter);
  }
  
  public char getDelimiter() {
    return this.userSettings.Delimiter;
  }
  
  public void setDelimiter(char delimiter) {
    this.userSettings.Delimiter = delimiter;
  }
  
  public char getRecordDelimiter() {
    return this.userSettings.RecordDelimiter;
  }
  
  public void setRecordDelimiter(char recordDelimiter) {
    this.useCustomRecordDelimiter = true;
    this.userSettings.RecordDelimiter = recordDelimiter;
  }
  
  public char getTextQualifier() {
    return this.userSettings.TextQualifier;
  }
  
  public void setTextQualifier(char textQualifier) {
    this.userSettings.TextQualifier = textQualifier;
  }
  
  public boolean getUseTextQualifier() {
    return this.userSettings.UseTextQualifier;
  }
  
  public void setUseTextQualifier(boolean useTextQualifier) {
    this.userSettings.UseTextQualifier = useTextQualifier;
  }
  
  public int getEscapeMode() {
    return this.userSettings.EscapeMode;
  }
  
  public void setEscapeMode(int escapeMode) {
    this.userSettings.EscapeMode = escapeMode;
  }
  
  public void setComment(char comment) {
    this.userSettings.Comment = comment;
  }
  
  public char getComment() {
    return this.userSettings.Comment;
  }
  
  public boolean getForceQualifier() {
    return this.userSettings.ForceQualifier;
  }
  
  public void setForceQualifier(boolean forceQualifier) {
    this.userSettings.ForceQualifier = forceQualifier;
  }
  
  public void write(String content, boolean preserveSpaces) throws IOException {
    // Byte code:
    //   0: aload_0
    //   1: invokespecial checkClosed : ()V
    //   4: aload_0
    //   5: invokespecial checkInit : ()V
    //   8: aload_1
    //   9: ifnonnull -> 15
    //   12: ldc ''
    //   14: astore_1
    //   15: aload_0
    //   16: getfield firstColumn : Z
    //   19: ifne -> 36
    //   22: aload_0
    //   23: getfield outputStream : Ljava/io/PrintWriter;
    //   26: aload_0
    //   27: getfield userSettings : Lcom/jiuqi/util/csv/CsvWriter$UserSettings;
    //   30: getfield Delimiter : C
    //   33: invokevirtual write : (I)V
    //   36: aload_0
    //   37: getfield userSettings : Lcom/jiuqi/util/csv/CsvWriter$UserSettings;
    //   40: getfield ForceQualifier : Z
    //   43: istore_3
    //   44: iload_2
    //   45: ifne -> 60
    //   48: aload_1
    //   49: invokevirtual length : ()I
    //   52: ifle -> 60
    //   55: aload_1
    //   56: invokevirtual trim : ()Ljava/lang/String;
    //   59: astore_1
    //   60: iload_3
    //   61: ifne -> 198
    //   64: aload_0
    //   65: getfield userSettings : Lcom/jiuqi/util/csv/CsvWriter$UserSettings;
    //   68: getfield UseTextQualifier : Z
    //   71: ifeq -> 198
    //   74: aload_1
    //   75: aload_0
    //   76: getfield userSettings : Lcom/jiuqi/util/csv/CsvWriter$UserSettings;
    //   79: getfield TextQualifier : C
    //   82: invokevirtual indexOf : (I)I
    //   85: iconst_m1
    //   86: if_icmpgt -> 196
    //   89: aload_1
    //   90: aload_0
    //   91: getfield userSettings : Lcom/jiuqi/util/csv/CsvWriter$UserSettings;
    //   94: getfield Delimiter : C
    //   97: invokevirtual indexOf : (I)I
    //   100: iconst_m1
    //   101: if_icmpgt -> 196
    //   104: aload_0
    //   105: getfield useCustomRecordDelimiter : Z
    //   108: ifne -> 131
    //   111: aload_1
    //   112: bipush #10
    //   114: invokevirtual indexOf : (I)I
    //   117: iconst_m1
    //   118: if_icmpgt -> 196
    //   121: aload_1
    //   122: bipush #13
    //   124: invokevirtual indexOf : (I)I
    //   127: iconst_m1
    //   128: if_icmpgt -> 196
    //   131: aload_0
    //   132: getfield useCustomRecordDelimiter : Z
    //   135: ifeq -> 153
    //   138: aload_1
    //   139: aload_0
    //   140: getfield userSettings : Lcom/jiuqi/util/csv/CsvWriter$UserSettings;
    //   143: getfield RecordDelimiter : C
    //   146: invokevirtual indexOf : (I)I
    //   149: iconst_m1
    //   150: if_icmpgt -> 196
    //   153: aload_0
    //   154: getfield firstColumn : Z
    //   157: ifeq -> 182
    //   160: aload_1
    //   161: invokevirtual length : ()I
    //   164: ifle -> 182
    //   167: aload_1
    //   168: iconst_0
    //   169: invokevirtual charAt : (I)C
    //   172: aload_0
    //   173: getfield userSettings : Lcom/jiuqi/util/csv/CsvWriter$UserSettings;
    //   176: getfield Comment : C
    //   179: if_icmpeq -> 196
    //   182: aload_0
    //   183: getfield firstColumn : Z
    //   186: ifeq -> 198
    //   189: aload_1
    //   190: invokevirtual length : ()I
    //   193: ifne -> 198
    //   196: iconst_1
    //   197: istore_3
    //   198: aload_0
    //   199: getfield userSettings : Lcom/jiuqi/util/csv/CsvWriter$UserSettings;
    //   202: getfield UseTextQualifier : Z
    //   205: ifeq -> 286
    //   208: iload_3
    //   209: ifne -> 286
    //   212: aload_1
    //   213: invokevirtual length : ()I
    //   216: ifle -> 286
    //   219: iload_2
    //   220: ifeq -> 286
    //   223: aload_1
    //   224: iconst_0
    //   225: invokevirtual charAt : (I)C
    //   228: istore #4
    //   230: iload #4
    //   232: bipush #32
    //   234: if_icmpeq -> 244
    //   237: iload #4
    //   239: bipush #9
    //   241: if_icmpne -> 246
    //   244: iconst_1
    //   245: istore_3
    //   246: iload_3
    //   247: ifne -> 286
    //   250: aload_1
    //   251: invokevirtual length : ()I
    //   254: iconst_1
    //   255: if_icmple -> 286
    //   258: aload_1
    //   259: aload_1
    //   260: invokevirtual length : ()I
    //   263: iconst_1
    //   264: isub
    //   265: invokevirtual charAt : (I)C
    //   268: istore #5
    //   270: iload #5
    //   272: bipush #32
    //   274: if_icmpeq -> 284
    //   277: iload #5
    //   279: bipush #9
    //   281: if_icmpne -> 286
    //   284: iconst_1
    //   285: istore_3
    //   286: iload_3
    //   287: ifeq -> 432
    //   290: aload_0
    //   291: getfield outputStream : Ljava/io/PrintWriter;
    //   294: aload_0
    //   295: getfield userSettings : Lcom/jiuqi/util/csv/CsvWriter$UserSettings;
    //   298: getfield TextQualifier : C
    //   301: invokevirtual write : (I)V
    //   304: aload_0
    //   305: getfield userSettings : Lcom/jiuqi/util/csv/CsvWriter$UserSettings;
    //   308: getfield EscapeMode : I
    //   311: iconst_2
    //   312: if_icmpne -> 374
    //   315: aload_1
    //   316: ldc '\'
    //   318: ldc '\\'
    //   320: invokestatic replace : (Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;)Ljava/lang/String;
    //   323: astore_1
    //   324: aload_1
    //   325: new java/lang/StringBuilder
    //   328: dup
    //   329: invokespecial <init> : ()V
    //   332: aload_0
    //   333: getfield userSettings : Lcom/jiuqi/util/csv/CsvWriter$UserSettings;
    //   336: getfield TextQualifier : C
    //   339: invokevirtual append : (C)Ljava/lang/StringBuilder;
    //   342: invokevirtual toString : ()Ljava/lang/String;
    //   345: new java/lang/StringBuilder
    //   348: dup
    //   349: ldc '\'
    //   351: invokespecial <init> : (Ljava/lang/String;)V
    //   354: aload_0
    //   355: getfield userSettings : Lcom/jiuqi/util/csv/CsvWriter$UserSettings;
    //   358: getfield TextQualifier : C
    //   361: invokevirtual append : (C)Ljava/lang/StringBuilder;
    //   364: invokevirtual toString : ()Ljava/lang/String;
    //   367: invokestatic replace : (Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;)Ljava/lang/String;
    //   370: astore_1
    //   371: goto -> 668
    //   374: aload_1
    //   375: new java/lang/StringBuilder
    //   378: dup
    //   379: invokespecial <init> : ()V
    //   382: aload_0
    //   383: getfield userSettings : Lcom/jiuqi/util/csv/CsvWriter$UserSettings;
    //   386: getfield TextQualifier : C
    //   389: invokevirtual append : (C)Ljava/lang/StringBuilder;
    //   392: invokevirtual toString : ()Ljava/lang/String;
    //   395: new java/lang/StringBuilder
    //   398: dup
    //   399: invokespecial <init> : ()V
    //   402: aload_0
    //   403: getfield userSettings : Lcom/jiuqi/util/csv/CsvWriter$UserSettings;
    //   406: getfield TextQualifier : C
    //   409: invokevirtual append : (C)Ljava/lang/StringBuilder;
    //   412: aload_0
    //   413: getfield userSettings : Lcom/jiuqi/util/csv/CsvWriter$UserSettings;
    //   416: getfield TextQualifier : C
    //   419: invokevirtual append : (C)Ljava/lang/StringBuilder;
    //   422: invokevirtual toString : ()Ljava/lang/String;
    //   425: invokestatic replace : (Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;)Ljava/lang/String;
    //   428: astore_1
    //   429: goto -> 668
    //   432: aload_0
    //   433: getfield userSettings : Lcom/jiuqi/util/csv/CsvWriter$UserSettings;
    //   436: getfield EscapeMode : I
    //   439: iconst_2
    //   440: if_icmpne -> 668
    //   443: aload_1
    //   444: ldc '\'
    //   446: ldc '\\'
    //   448: invokestatic replace : (Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;)Ljava/lang/String;
    //   451: astore_1
    //   452: aload_1
    //   453: new java/lang/StringBuilder
    //   456: dup
    //   457: invokespecial <init> : ()V
    //   460: aload_0
    //   461: getfield userSettings : Lcom/jiuqi/util/csv/CsvWriter$UserSettings;
    //   464: getfield Delimiter : C
    //   467: invokevirtual append : (C)Ljava/lang/StringBuilder;
    //   470: invokevirtual toString : ()Ljava/lang/String;
    //   473: new java/lang/StringBuilder
    //   476: dup
    //   477: ldc '\'
    //   479: invokespecial <init> : (Ljava/lang/String;)V
    //   482: aload_0
    //   483: getfield userSettings : Lcom/jiuqi/util/csv/CsvWriter$UserSettings;
    //   486: getfield Delimiter : C
    //   489: invokevirtual append : (C)Ljava/lang/StringBuilder;
    //   492: invokevirtual toString : ()Ljava/lang/String;
    //   495: invokestatic replace : (Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;)Ljava/lang/String;
    //   498: astore_1
    //   499: aload_0
    //   500: getfield useCustomRecordDelimiter : Z
    //   503: ifeq -> 556
    //   506: aload_1
    //   507: new java/lang/StringBuilder
    //   510: dup
    //   511: invokespecial <init> : ()V
    //   514: aload_0
    //   515: getfield userSettings : Lcom/jiuqi/util/csv/CsvWriter$UserSettings;
    //   518: getfield RecordDelimiter : C
    //   521: invokevirtual append : (C)Ljava/lang/StringBuilder;
    //   524: invokevirtual toString : ()Ljava/lang/String;
    //   527: new java/lang/StringBuilder
    //   530: dup
    //   531: ldc '\'
    //   533: invokespecial <init> : (Ljava/lang/String;)V
    //   536: aload_0
    //   537: getfield userSettings : Lcom/jiuqi/util/csv/CsvWriter$UserSettings;
    //   540: getfield RecordDelimiter : C
    //   543: invokevirtual append : (C)Ljava/lang/StringBuilder;
    //   546: invokevirtual toString : ()Ljava/lang/String;
    //   549: invokestatic replace : (Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;)Ljava/lang/String;
    //   552: astore_1
    //   553: goto -> 574
    //   556: aload_1
    //   557: ldc '\\r'
    //   559: ldc '\\\r'
    //   561: invokestatic replace : (Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;)Ljava/lang/String;
    //   564: astore_1
    //   565: aload_1
    //   566: ldc '\\n'
    //   568: ldc '\\\n'
    //   570: invokestatic replace : (Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;)Ljava/lang/String;
    //   573: astore_1
    //   574: aload_0
    //   575: getfield firstColumn : Z
    //   578: ifeq -> 668
    //   581: aload_1
    //   582: invokevirtual length : ()I
    //   585: ifle -> 668
    //   588: aload_1
    //   589: iconst_0
    //   590: invokevirtual charAt : (I)C
    //   593: aload_0
    //   594: getfield userSettings : Lcom/jiuqi/util/csv/CsvWriter$UserSettings;
    //   597: getfield Comment : C
    //   600: if_icmpne -> 668
    //   603: aload_1
    //   604: invokevirtual length : ()I
    //   607: iconst_1
    //   608: if_icmple -> 645
    //   611: new java/lang/StringBuilder
    //   614: dup
    //   615: ldc '\'
    //   617: invokespecial <init> : (Ljava/lang/String;)V
    //   620: aload_0
    //   621: getfield userSettings : Lcom/jiuqi/util/csv/CsvWriter$UserSettings;
    //   624: getfield Comment : C
    //   627: invokevirtual append : (C)Ljava/lang/StringBuilder;
    //   630: aload_1
    //   631: iconst_1
    //   632: invokevirtual substring : (I)Ljava/lang/String;
    //   635: invokevirtual append : (Ljava/lang/String;)Ljava/lang/StringBuilder;
    //   638: invokevirtual toString : ()Ljava/lang/String;
    //   641: astore_1
    //   642: goto -> 668
    //   645: new java/lang/StringBuilder
    //   648: dup
    //   649: ldc '\'
    //   651: invokespecial <init> : (Ljava/lang/String;)V
    //   654: aload_0
    //   655: getfield userSettings : Lcom/jiuqi/util/csv/CsvWriter$UserSettings;
    //   658: getfield Comment : C
    //   661: invokevirtual append : (C)Ljava/lang/StringBuilder;
    //   664: invokevirtual toString : ()Ljava/lang/String;
    //   667: astore_1
    //   668: aload_0
    //   669: getfield outputStream : Ljava/io/PrintWriter;
    //   672: aload_1
    //   673: invokevirtual write : (Ljava/lang/String;)V
    //   676: iload_3
    //   677: ifeq -> 694
    //   680: aload_0
    //   681: getfield outputStream : Ljava/io/PrintWriter;
    //   684: aload_0
    //   685: getfield userSettings : Lcom/jiuqi/util/csv/CsvWriter$UserSettings;
    //   688: getfield TextQualifier : C
    //   691: invokevirtual write : (I)V
    //   694: aload_0
    //   695: iconst_0
    //   696: putfield firstColumn : Z
    //   699: return
    // Line number table:
    //   Java source line number -> byte code offset
    //   #273	-> 0
    //   #275	-> 4
    //   #277	-> 8
    //   #278	-> 12
    //   #281	-> 15
    //   #282	-> 22
    //   #285	-> 36
    //   #287	-> 44
    //   #288	-> 55
    //   #291	-> 60
    //   #292	-> 64
    //   #293	-> 74
    //   #294	-> 89
    //   #295	-> 104
    //   #296	-> 112
    //   #297	-> 122
    //   #298	-> 131
    //   #299	-> 139
    //   #300	-> 153
    //   #301	-> 168
    //   #304	-> 182
    //   #305	-> 196
    //   #308	-> 198
    //   #309	-> 212
    //   #310	-> 223
    //   #312	-> 230
    //   #313	-> 244
    //   #316	-> 246
    //   #317	-> 258
    //   #319	-> 270
    //   #320	-> 284
    //   #325	-> 286
    //   #326	-> 290
    //   #328	-> 304
    //   #329	-> 315
    //   #331	-> 324
    //   #332	-> 325
    //   #334	-> 354
    //   #331	-> 367
    //   #336	-> 374
    //   #337	-> 375
    //   #338	-> 402
    //   #339	-> 412
    //   #336	-> 425
    //   #341	-> 432
    //   #342	-> 443
    //   #344	-> 452
    //   #345	-> 482
    //   #344	-> 495
    //   #347	-> 499
    //   #348	-> 506
    //   #349	-> 514
    //   #350	-> 536
    //   #348	-> 549
    //   #352	-> 556
    //   #354	-> 565
    //   #358	-> 574
    //   #359	-> 588
    //   #360	-> 603
    //   #361	-> 611
    //   #362	-> 620
    //   #361	-> 638
    //   #364	-> 645
    //   #365	-> 654
    //   #364	-> 664
    //   #370	-> 668
    //   #372	-> 676
    //   #373	-> 680
    //   #376	-> 694
    //   #377	-> 699
    // Local variable table:
    //   start	length	slot	name	descriptor
    //   0	700	0	this	Lcom/jiuqi/util/csv/CsvWriter;
    //   0	700	1	content	Ljava/lang/String;
    //   0	700	2	preserveSpaces	Z
    //   44	656	3	textQualify	Z
    //   230	56	4	firstLetter	C
    //   270	16	5	lastLetter	C
  }
  
  public void write(String content) throws IOException {
    write(content, false);
  }
  
  public void writeComment(String commentText) throws IOException {
    checkClosed();
    checkInit();
    this.outputStream.write(this.userSettings.Comment);
    this.outputStream.write(commentText);
    if (this.useCustomRecordDelimiter) {
      this.outputStream.write(this.userSettings.RecordDelimiter);
    } else {
      this.outputStream.println();
    } 
    this.firstColumn = true;
  }
  
  public void writeRecord(String[] values, boolean preserveSpaces) throws IOException {
    if (values != null && values.length > 0) {
      for (int i = 0; i < values.length; i++)
        write(values[i], preserveSpaces); 
      endRecord();
    } 
  }
  
  public void writeRecord(String[] values) throws IOException {
    writeRecord(values, false);
  }
  
  public void endRecord() throws IOException {
    checkClosed();
    checkInit();
    if (this.useCustomRecordDelimiter) {
      this.outputStream.write(this.userSettings.RecordDelimiter);
    } else {
      this.outputStream.println();
    } 
    this.firstColumn = true;
  }
  
  private void checkInit() throws IOException {
    if (!this.initialized) {
      if (this.fileName != null)
        this.outputStream = new PrintWriter(new OutputStreamWriter(
              new FileOutputStream(this.fileName), this.charset)); 
      this.initialized = true;
    } 
  }
  
  public void flush() {
    this.outputStream.flush();
  }
  
  public void close() {
    if (!this.closed) {
      close(true);
      this.closed = true;
    } 
  }
  
  private void close(boolean closing) {
    if (!this.closed) {
      if (closing)
        this.charset = null; 
      try {
        if (this.initialized)
          this.outputStream.close(); 
      } catch (Exception exception) {}
      this.outputStream = null;
      this.closed = true;
    } 
  }
  
  private void checkClosed() throws IOException {
    if (this.closed)
      throw new IOException(
          "This instance of the CsvWriter class has already been closed."); 
  }
  
  protected void finalize() {
    close(false);
  }
  
  public static String replace(String original, String pattern, String replace) {
    int len = pattern.length();
    int found = original.indexOf(pattern);
    if (found > -1) {
      StringBuffer sb = new StringBuffer();
      int start = 0;
      while (found != -1) {
        sb.append(original.substring(start, found));
        sb.append(replace);
        start = found + len;
        found = original.indexOf(pattern, start);
      } 
      sb.append(original.substring(start));
      return sb.toString();
    } 
    return original;
  }
}
